#ifndef FLYFM_CHECKSMBSHAREAUTHJOB_H
#define FLYFM_CHECKSMBSHAREAUTHJOB_H

#include <QSharedPointer>

#include <flyfm/job.h>

namespace FlyFM
{

class Authenticator;

class CheckSmbShareAuthJob : public FlyFM::Job
{
    Q_OBJECT
public:
    enum Error { NoError = 0, NotSambaEntry, LoginCancelled, NoShareEntry, NotInFuseGroup };

    CheckSmbShareAuthJob(QSharedPointer<Authenticator> authenticator, const QString& share, const QString& host, const QString& workgroup = QString(), QObject* parent = 0);
    ~CheckSmbShareAuthJob();

    void start() Q_DECL_OVERRIDE;

    QString share() const;
    QString host() const;
    QString workgroup() const;

    QString hostForKerberos() const;

private:
    enum AuthResult { AuthAccessGranted, AuthAccessDenied, AuthLogonFailure };
    AuthResult checkKerberosAuth();
    AuthResult checkAuth(const QString& user, const QString& password) const;
    QString resourcePath() const;

    void setAuthForShare(const QString& share, const QString& host, const QString& workgroup, const QString& user, const QString& password);
    bool getAuthForShare(const QString& share, const QString& host, const QString& workgroup, QString* user, QString* password) const;

    QSharedPointer<Authenticator> m_authenticator;
    QString m_hostForKerberos;
};

} // namespace FlyFM

#endif // FLYFM_CHECKSMBSHAREAUTHJOB_H
