/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMCORE_H
#define FLYFMCORE_H

#include <QMap>
#include <QStringList>
#include <QVariant>

class QWidget;

namespace FlyFM
{

namespace Core
{

enum HaltReason { Exited = 0, Interrupted = 1, Segfaulted = 2 };

typedef void(*typedef_ReleaseFunc)();
typedef void(*typedef_HaltHandlerFunc)(HaltReason);

void initialize();
void release();

bool isReleased();

bool installTranslator(const QString &name);
void moveForCenter(QWidget *);
void moveForPresentation(QWidget *);

QString md5HashForString(const QString &string);
QString dataSizeAsString(long long size, int mul = 1); // Convert byte size to Kb, Mb, Gb
QStringList allMimeTypes();
QString userName();
QString version();
QString versionedName();

void registerReleaseFunction(typedef_ReleaseFunc);
void unregisterReleaseFunction(typedef_ReleaseFunc);

void registerHaltHandlerFunction(typedef_HaltHandlerFunc);
void unregisterHaltHandlerFunction(typedef_HaltHandlerFunc);

QMap<QString, QString> fromStringToScreenMap(const QString &string);
QString fromScreenMapToString(const QMap<QString, QString> &map);

int compareVariants(const QVariant& v1, const QVariant& v2);

bool startProcess(const QString &command);
bool startProcess(const QString &program, const QStringList &arguments, const QString & workingDirectory = QString());

}

}

#endif // FLYFMCORE_H
