#ifndef FLYFMDESKTOPVIEW_H
#define FLYFMDESKTOPVIEW_H

#include <QListView>
#include <QImage>

namespace FlyFM
{

class Entry;
class EntryList;

class DesktopView : public QListView
{
    Q_OBJECT
public:
    DesktopView(QWidget *parent = 0);
    ~DesktopView();

    virtual EntryList selectedEntries();
signals:
    void customPopupContextMenuRequested(const QPoint &, const QModelIndexList &);
    void dropContextMenuRequested(const QPoint &, const EntryList &, const Entry &);
    void openActionRequested(const QModelIndexList &);
    void upActionRequested();

public slots:
    void reset() Q_DECL_OVERRIDE;

protected slots:
    void openActionEmitter();

    void customPopupContextMenu(const QPoint &);
    void possibleEntryPosition(const EntryList &, const QPoint &);
    void layoutChanged();

protected:
    void initializeBackground();
    void clearItemPositions();
    void pushItemPositions();
    void popItemPositions();

    void dropEvent(QDropEvent *event) Q_DECL_OVERRIDE;
    void paintEvent(QPaintEvent *event) Q_DECL_OVERRIDE;
    void mousePressEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
    void dragMoveEvent(QDragMoveEvent *) Q_DECL_OVERRIDE;
    void wheelEvent(QWheelEvent *) Q_DECL_OVERRIDE;

    QModelIndex m_overIndex;
    QImage m_desktopImage;

    friend class DesktopWidget;
    friend class PopupMenu;
};
}

#endif // FLYFMDESKTOPVIEW_H
