/*

  Core library
  Entry list class

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_ENTRYLIST_H
#define FLYFM_ENTRYLIST_H

#include <QList>
#include <QMetaType>

#include <flyfm/entry.h>

class QDBusArgument;
class QStringList;

namespace FlyFM
{
class EntryList : public QList<Entry>
{
public:

    EntryList();
    EntryList(const Entry &entry);
    EntryList(const QList<Entry> &list);
    virtual ~EntryList();

    /*static QByteArray toData(const EntryList &entries);
    static EntryList fromData(const QByteArray &data);*/

    bool hasPlace() const;
    bool hasPurePlace() const;
    bool hasDir() const;
    bool hasFile() const;
    bool hasUnmountedStoragePlace() const;

    EntryList places() const;
    EntryList dirs() const;
    EntryList files() const;

    EntryList sublistOfType(int type) const;
    bool hasChildOfType(int type) const;

    EntryList mergeWith(const EntryList &entries);

    QStringList paths() const;
    QStringList localPaths() const;
    QStringList localPathsWithScheme() const;

    void sort(bool ascending = true);

    bool containsSimilar(const Entry &entry) const;
};
typedef QListIterator<Entry> EntryListIterator;
}

Q_DECLARE_METATYPE(FlyFM::EntryList)

QDBusArgument &operator<<(QDBusArgument &arg, const FlyFM::EntryList &entryList);
const QDBusArgument &operator>>(const QDBusArgument &arg, FlyFM::EntryList &entryList);

#endif // FLYFMENTRYLIST_H
