/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMENTRYWATCHER_H
#define FLYFMENTRYWATCHER_H

#include <QObject>
#include <QStringList>
#include <QMap>
#include <QSet>

namespace FlyFM
{

class Entry;
class EntryList;
class InotifyWatcher;

class EntryWatcher : public QObject
{
    Q_OBJECT
public:
    EntryWatcher(QObject *parent);
    virtual ~EntryWatcher();

    void addEntry(const Entry &entry);
    void removeEntry(const Entry &entry);
    void setDisabled(bool disabled);
    void clear();

    void notifyEntryCreated(const Entry& entry);
    void notifyEntryDeleted(const Entry& entry);
    void notifyEntryChanged(const Entry& entry);

signals:
    void entryCreated(const Entry& entry);
    void entryDeleted(const Entry& entry);
    void entryChanged(const Entry& entry);
    void entryAttributesChanged(const Entry& entry);
    void entryReset(const Entry& entry);

protected slots:
    void entryChangedNotify(const Entry &entry, bool anyWay = false);

    void unregisterPath(const QString& path);

    void slotPathCreated(const QString& path, const QString& fileName);
    void slotPathDeleted(const QString& path, const QString& fileName);
    void slotPathChanged(const QString& path, const QString& fileName);
    void slotPathAttributesChanged(const QString& path, const QString& fileName);

protected:
    void notifyPath(const char* signal, const QString& path, const QString& fileName);
    bool isRefreshable(const QString &localPath) const;

    InotifyWatcher* inotifyWatcher;
    QMap<QString, QSet<Entry> > m_pathToPlaceRelatedPathsMap;
    bool m_isDisabled;
    QStringList m_unrefreshableRoots;

    friend class VFS;
};
}

#endif // FLYFMENTRYWATCHER_H
