/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMFOPS_H
#define FLYFMFOPS_H

#include <QMutex>
#include <QMap>

#include <flyfm/entry.h>
#include <flyfm/serviceinterfacesingletone.h>

namespace FlyFM
{

class EntryList;

class FOpId
{
public:
    FOpId();
    FOpId(const FOpId &id);
    ~FOpId();

    FOpId& operator=(const FOpId &id);
    bool operator<(const FOpId &id) const;
    bool operator==(const FOpId &id) const;

private:
    FOpId(const QString &op);
    void setStamp(const QString &);
    QString stamp() const;

    QString m_stamp;

    friend class FOps;
};

class FOpRes
{
public:
    FOpRes();
    FOpRes(const FOpRes &res);
    ~FOpRes();

    bool isNull() const;
    QString text() const;
    int status() const;

    FOpRes& operator=(const FOpRes &res);

private:
    FOpRes(const QString &text, int status);

    bool m_isNull;
    QString m_text;
    int m_status;

    friend class FOps;
};

class FOps : public QObject, public ServiceInterfaceSingletone
{
    Q_OBJECT
public:
    static FOps *ptr();
    ~FOps();

    FOpId makeDir(const Entry &entry);
    FOpId makeFile(const Entry &entry);
    FOpId makeShortcut(const Entry &entry);
    QStringList fileTemplates() const;
    FOpId makeFromTemplate(const Entry& entry, const QString& templatePath);
    FOpId makeShortcuts(const EntryList &entryList, const Entry &dest = Entry());
    FOpId copy(const EntryList &srcs, const Entry &dest);
    FOpId rename(const Entry &source, const Entry &destination);
    FOpId remove(const EntryList &entries, bool force = false);
    FOpId move(const EntryList &srcs, const Entry &dest);
    FOpId trash(const EntryList &entries);
//      FOpId restore( const EntryList &entries );
    FOpId restore(const Entry &entry);
    FOpId cleanTrash();

    int operationProgress(const FOpId &opId) const;

    FOpRes popOperationResult(const FOpId &opId, bool sync = false);

    static bool isTrash(const Entry &entry);
    static bool isTrashEntries(const EntryList &entries);
    static bool isEmptyTrash();

signals:
    void operationFinished(const FlyFM::FOpId &, int);

protected slots:
    void finished(const QString &, int, const QString &);

protected:
    FOps(QObject *parent = 0);

private:
    QMutex m_resMapMutex;

    static FOps *m_ptr;
    QMap<FOpId, FOpRes> m_opIdResMap;

    QWeakPointer<QDBusInterface> m_interface;
};

}

#endif // FLYFMFOPS_H
