/*

  Graphic user's interface library
  Navigation control

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMNAVIGATION_H
#define FLYFMNAVIGATION_H

#include <QObject>

#include <flyfm/entrylist.h>

namespace FlyFM
{
class Navigation : public QObject
{
    Q_OBJECT
public:
    Navigation(QObject *parent = 0);
    virtual ~Navigation();

    Entry current() const;
    Entry previous() const;
    Entry validParent() const;
    void setLockedEntries(const EntryList &entries);

    EntryList backList() const;
    EntryList forwardList() const;

    bool isBackEnabled() const;
    bool isForwardEnabled() const;
    bool isUpEnabled() const;

signals:
    void backEnabled(bool flag);
    void forwardEnabled(bool flag);
    void upEnabled(bool flag);

    void currentChanged(const Entry &entry);
    void backListUpdated(const EntryList &list);
    void forwardListUpdated(const EntryList &list);

    void highlightEntry(const Entry& entry);

public slots:
    bool setCurrent(const Entry &entry);

    virtual void back(int count = 1);
    virtual void forward(int count = 1);
    virtual void up();

protected:
    void emitEnableSignals();

private:
    bool isValidCurrentIndex() const;

private:
    int m_current;
    EntryList m_list;
    EntryList m_lockedEntries;
};
}

#endif // FLYFMNAVIGATION_H
