#ifndef SMBPASSWORDSJOBS_H
#define SMBPASSWORDSJOBS_H

#include <QMap>
#include <QString>

#include <flyjob.h>
#include <flysecretsserviceitem.h>

namespace FlyFM
{

class FindNetPasswordJob : public FlyJob
{
    Q_OBJECT
public:
    FindNetPasswordJob(const QString& protocol, const QString& domain, const QString& server, const QString& object, QObject *parent = 0);

    void start() Q_DECL_OVERRIDE;

    typedef QExplicitlySharedDataPointer< FlySecretsService::SecretItem > Item;
    typedef QList< Item > ItemList;

    QMap<QString, QString> logins() const;

private slots:
    void slotSearchJobDone(FlyJob*job);

private:
    QHash<Item, QString> filterItems(const ItemList& items);

private:
    QString m_protocol;
    QString m_domain;
    QString m_server;
    QString m_object;

    QMap<QString, QString> m_logins;
};

class SetNetPasswordJob : public FlyJob
{
    Q_OBJECT
public:
    SetNetPasswordJob(const QString& protocol,
                      const QString& domain,
                      const QString& server,
                      const QString& object,
                      const QString& user,
                      const QString& password,
                      QObject *parent = 0);

    void start() Q_DECL_OVERRIDE;

private slots:
    void slotCreateItemJobDone(FlyJob*job);

private:
    QString m_protocol;
    QString m_domain;
    QString m_server;
    QString m_object;
    QString m_user;
    QString m_password;
};

}

#endif // SMBPASSWORDSJOBS_H
