#ifndef FLYFM_NETWORKPLACE_H
#define FLYFM_NETWORKPLACE_H

#include <QString>
#include <QList>
#include <QMetaType>

class QDataStream;
class QDBusArgument;

namespace FlyFM
{
class NetworkPlace;
typedef QList<NetworkPlace> NetworkPlaceList;
}

QDataStream& operator<<(QDataStream &out, const FlyFM::NetworkPlace &place);
QDataStream& operator>>(QDataStream &in, FlyFM::NetworkPlace &place);

QDBusArgument &operator<<(QDBusArgument &arg, const FlyFM::NetworkPlace &NetworkPlace);
const QDBusArgument &operator>>(const QDBusArgument &arg, FlyFM::NetworkPlace &NetworkPlace);

namespace FlyFM
{

class NetworkPlace
{
public:
    NetworkPlace();
    NetworkPlace(const QString& name, const QString& url, const QString& localPath = QString());

    bool operator==(const NetworkPlace &place) const;

    bool isNull() const;

    QString name() const;
    QString url() const;
    QString localPath() const;
    void setLocalPath(const QString& path);

private:
    friend QDataStream& ::operator<<(QDataStream &out, const NetworkPlace &place);
    friend QDataStream& ::operator>>(QDataStream &in, NetworkPlace &place);

    QDBusArgument& toDBusArgument(QDBusArgument &arg) const;
    const QDBusArgument& fromDBusArgument(const QDBusArgument &arg);

    friend QDBusArgument& ::operator<<(QDBusArgument &arg, const NetworkPlace &NetworkPlace);
    friend const QDBusArgument& ::operator>>(const QDBusArgument &arg, NetworkPlace &NetworkPlace);

    QString m_name;
    QString m_url;
    QString m_localPath;
};

}

Q_DECLARE_METATYPE(FlyFM::NetworkPlace)
Q_DECLARE_METATYPE(FlyFM::NetworkPlaceList)

#endif // FLYFM_NETWORKPLACE_H
