/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMPATHANALYZER_H
#define FLYFMPATHANALYZER_H

#include <QString>
#include <QQueue>

#include <flyfm/entry.h>
#include <flyfm/entrylist.h>
#include <flyfm/pathanalyzerpair.h>

namespace FlyFM
{

class PathAnalyzer
{
public:
    enum Error { NoError = 0, DirToFileError = 1, EntriesToFileError = 2 };
    enum Mode { DefaultMode = 0, DirectMode = 1 };

    explicit PathAnalyzer(const EntryList &sources = EntryList(), const Entry &destination = Entry(), Mode mode = DefaultMode);
    ~PathAnalyzer();

    void resolve(const EntryList &sources, const Entry &destination = Entry(), Mode mode = DefaultMode);
    void stopResolving();

    void removeChilds(const Entry &entry);
    void clear();

    EntryList sources() const;
    Entry destination() const;

    Error error() const;
    QString errorText() const;

    bool isSourceEntry(const Entry &entry) const;

    void prepend(const PathAnalyzerPair &pair);
    PathAnalyzerPair getNext();
    void removeNext();
    quint64 totalSize() const;
    int count(bool fileOnly = false) const;
    bool isEmpty() const;

    static bool isValid(const EntryList &sources);

protected:
    bool hasToStopResolving();

    EntryList m_sources;
    Entry m_destination;
    QQueue<PathAnalyzerPair> m_sequence;
    quint64 m_totalSize;

    bool m_isSingleSourceMode;
    bool m_isSingleDir;

    Error m_error;
    Mode m_mode;
    QString m_errorText;

    QAtomicInt m_dontDoResolve;
    QAtomicInt m_gettingChildsFromVFS;

    friend class PathAnalyzerPair;
};

}

#endif // FLYFMPATHANALYZER_H
