/*

  Graphic user's interface library
  Places widget

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_PLACESWIDGET_H
#define FLYFM_PLACESWIDGET_H

#include <flyfm/treeview.h>
#include <flyfm/imanageiconsize.h>

class QListWidgetItem;

namespace FlyFM
{

class Entry;
class EntryList;
class Navigation;
class PlacesModel;
class IconSizeActionsProvider;
class ItemViewTouchGesturesManager;

class PlacesWidget : public ScheduledUpdateTreeView, public IManageIconSize
{
    Q_OBJECT
public:
    PlacesWidget(QWidget *parent = 0);
    ~PlacesWidget();

    void setNavigation(Navigation* navigation);

    IconSize iconSize() const Q_DECL_OVERRIDE;

    QSize sizeHint() const Q_DECL_OVERRIDE;

    static QByteArray defaultState(bool tabletMode);
    QByteArray saveState() const;
    void restoreState(const QByteArray& state);

    const Entry rootEntry() const;

    void setKineticScrollingEnabled(bool on);
    void setPinchZoomEnabled(bool on);

    PlacesModel* model();
    const PlacesModel* model() const;

    void expandFavourites();

public slots:
    void setIconSize(IconSize size) Q_DECL_OVERRIDE;
    void setCurrentEntry(const Entry& entry);

signals:
    void entrySelected(const Entry&);

protected slots:
    void dropPopupContextMenu(const QPoint &, const EntryList &, const Entry &);
    void customPopupContextMenu(const QPoint &, const QModelIndexList &);
    void checkNewSelection(const QItemSelection& selection);
    void activatedAction(const QModelIndex&);

    void increaseIconSizeAction();
    void decreaseIconSizeAction();

protected:
    bool viewportEvent(QEvent *event) Q_DECL_OVERRIDE;

    void openAction(const QModelIndex &index);

    QMap<QListWidgetItem *, Entry> m_itemEntryMap;

    PlacesModel *m_model;
    IconSize m_iconSize;
    Navigation *m_navigation;

    IconSizeActionsProvider* m_iconSizeActionsProvider;

    ItemViewTouchGesturesManager* m_touchGesturesManager;
};

}

#endif // FLYFM_PLACESWIDGET_H
