/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMPROPERTYDATAHOLDER_H
#define FLYFMPROPERTYDATAHOLDER_H

#include <QHash>
#include <QVariant>
#include <QReadWriteLock>
#include <QVector>
#include <QBitArray>

namespace FlyFM
{

class Entry;
class PropertyEnumerator;
class EntryPropertySet;

class PropertyDataHolder
{
public:
    QVariant propertyValueData(const Entry& entry, int index, int role, const PropertyEnumerator *propertyEnumerator);

    void invalidate(int sortIndex, const PropertyEnumerator *propertyEnumerator);
    void updatePresentation(const Entry& entry);
    void delayedRefineAllProperties(const Entry &entry, PropertyEnumerator* propertyEnumerator);
    void refineAllProperties(const Entry &entry, PropertyEnumerator* propertyEnumerator);

    void refinePropertyAtIndex(int index, const PropertyEnumerator* propertyEnumerator);

    bool isRefined() const;

protected:
    PropertyDataHolder(int nPropertySets);
    virtual ~PropertyDataHolder();

    void incrementReferenceCount();
    bool decrementReferenceCount();

    void setRefined();

    EntryPropertySet* preparePropertySet(const Entry& entry, int setIndex, const PropertyEnumerator *propertyEnumerator);

    QVector<EntryPropertySet*> m_propertySets;
    QBitArray m_isPropertySetInited;
    QBitArray m_isPropertySetRefined;

    QAtomicInt m_refCount;

    friend class PropertyEnumerator;
};

typedef QList<PropertyDataHolder *> PropertyDataHolderPtrList;
}

#endif // FLYFMROPERTYDATAHOLDER_H
