/*

  File system entry properties library
  Plugin abstract class

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMPROPSPLUGIN_H
#define FLYFMPROPSPLUGIN_H

#include <QtPlugin>

#include <flyfm/plugin.h>
#include <flyfm/entrylist.h>

namespace FlyFM
{

class PropsPlugin;
class PropsPluginWidget;
class FindPluginWidget;
class FilterPluginWidget;
class EntryPropertySet;

typedef QList<PropsPlugin*> PropsPluginPtrList;

class PropsPlugin : public Plugin
{
public:
    PropsPlugin();
    virtual ~PropsPlugin();

    virtual QString label() = 0;
    virtual int orderPriority();

    virtual bool hasTextDigest();
    virtual QString textDigest(const EntryList &entries, bool *m_pAliveFlag = 0);

    virtual bool hasFindWidget();
    virtual bool hasPropsWidgetFor(const EntryList &);

    virtual QList<FindPluginWidget*> createFindWidgets(QWidget *parent = 0);
    virtual QList<FilterPluginWidget*> createFilterWidgets(QWidget *parent = 0);
    virtual PropsPluginWidget *createPropsWidget(const EntryList &entries, QWidget *parent = 0);

    virtual int propertySetSize() const;
    virtual QVariant propertyHeaderData(int index, int role);
    virtual bool isPropertySortable(int index);
    virtual EntryPropertySet* createPropertySet() const;
};

}

Q_DECLARE_INTERFACE(FlyFM::PropsPlugin, "ru.rusbitech.fly-fm.PropsPlugin/1.0")

#endif // FLYFMPROPSPLUGIN_H
