#ifndef SCHEDULEDUPDATEVIEW_H
#define SCHEDULEDUPDATEVIEW_H

#include <QObject>
#include <QPointer>
#include <QMap>
#include <QQueue>
#include <QModelIndex>

class QTimer;
class QShowEvent;
class QResizeEvent;
class QAbstractItemView;

namespace FlyFM
{

class ScheduledViewUpdater;

template <class T>
class ScheduledUpdateView : public T
{
public:
    ScheduledUpdateView(QWidget* parent = 0);

    void scheduleUpdating();

    void setModel(QAbstractItemModel * model) Q_DECL_OVERRIDE;
protected:
    void scrollContentsBy(int dx, int dy) Q_DECL_OVERRIDE;
    void showEvent(QShowEvent*) Q_DECL_OVERRIDE;
    void resizeEvent(QResizeEvent* event) Q_DECL_OVERRIDE;

    void rowsAboutToBeRemoved(const QModelIndex & parent, int start, int end) Q_DECL_OVERRIDE;
    void rowsInserted(const QModelIndex & parent, int start, int end) Q_DECL_OVERRIDE;
private:
    ScheduledViewUpdater* m_viewUpdater;
};

class ScheduledViewUpdater : public QObject
{
    Q_OBJECT
public:
    ScheduledViewUpdater(QAbstractItemView* view);
    ~ScheduledViewUpdater();

    void scheduleUpdating();
    void setModel(QAbstractItemModel * model);

private slots:
    void updateVisibleIndexes();
    void updatePendingIndexes();

    void layoutChanged();
private:
    QTimer* m_updateTimer;
    QQueue<QPersistentModelIndex> pendingIndexes;
    QPointer<QAbstractItemView> m_view;
};

template <class T>
ScheduledUpdateView<T>::ScheduledUpdateView(QWidget* parent)
    : T(parent)
{
    m_viewUpdater = new ScheduledViewUpdater(this);
}

template <class T>
void ScheduledUpdateView<T>::scheduleUpdating()
{
    m_viewUpdater->scheduleUpdating();
}

template <class T>
void ScheduledUpdateView<T>::setModel(QAbstractItemModel* newModel)
{
    m_viewUpdater->setModel(newModel);

    T::setModel(newModel);
}

template <class T>
void ScheduledUpdateView<T>::scrollContentsBy(int dx, int dy)
{
    T::scrollContentsBy(dx, dy);
    scheduleUpdating();
}

template <class T>
void ScheduledUpdateView<T>::showEvent(QShowEvent* e)
{
    T::showEvent(e);
    scheduleUpdating();
}

template <class T>
void ScheduledUpdateView<T>::resizeEvent(QResizeEvent * event)
{
    T::resizeEvent(event);
    scheduleUpdating();
}

template <class T>
void ScheduledUpdateView<T>::rowsAboutToBeRemoved(const QModelIndex & parent, int start, int end)
{
    T::rowsAboutToBeRemoved(parent, start, end);
    scheduleUpdating();
}

template <class T>
void ScheduledUpdateView<T>::rowsInserted(const QModelIndex & parent, int start, int end)
{
    T::rowsInserted(parent, start, end);
    scheduleUpdating();
}

}

#endif // SCHEDULEDUPDATEVIEW_H
