/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMSTORAGEPLACE_H
#define FLYFMSTORAGEPLACE_H

#include <QString>
#include <QMetaType>
#include <QList>

class QDBusArgument;

namespace FlyFM
{
class StoragePlace;
typedef QList<StoragePlace> StoragePlaceList;
}

QDBusArgument &operator<<(QDBusArgument &arg, const FlyFM::StoragePlace &storagePlace);
const QDBusArgument &operator>>(const QDBusArgument &arg, FlyFM::StoragePlace &storagePlace);

namespace FlyFM
{

class StoragePlace
{
public:
    enum Type { Undefined = 0, FloppyDrive = 1, CdromDrive = 2, UsbDisk = 3, HardDisk = 4, SdMmc = 5, MemoryStick = 6, CompactFlash = 7, SmartMedia = 8, Xd = 9, Network = 10 };

    StoragePlace();
    virtual ~StoragePlace();

    bool operator<(const StoragePlace &storagePlace) const;
    bool operator==(const StoragePlace &storagePlace) const;

    bool isValid() const;

    bool isEjectable() const;
    bool isMounted() const;
    bool isMountable() const;

    QString mountPoint() const;
    QString blockDevice() const;
    QString name() const;
    QString labelName() const;
    QString iconName() const;
    Type type() const;

    QString udi() const;

protected:
    QDBusArgument& toDBusArgument(QDBusArgument &arg) const;
    const QDBusArgument& fromDBusArgument(const QDBusArgument &arg);

    friend QDBusArgument& ::operator<<(QDBusArgument &arg, const StoragePlace &storagePlace);
    friend const QDBusArgument& ::operator>>(const QDBusArgument &arg, StoragePlace &storagePlace);

    QString m_mountPoint;
    QString m_blockDevice;
    bool m_isEjectable;
    bool m_isMounted;
    bool m_isMountable;
    QString m_name;
    QString m_labelName;
    QString m_iconName;

    QString m_udi;
    Type m_type;

    friend class StoragePlacesManager;
    friend class StoragePlacesManagerPrivate;
};

}

Q_DECLARE_METATYPE(FlyFM::StoragePlace)
Q_DECLARE_METATYPE(FlyFM::StoragePlaceList)

#endif // FLYFMSTORAGEPLACE_H
