#ifndef FLYFM_TREEVIEWMOD_H
#define FLYFM_TREEVIEWMOD_H

#include <QTreeView>

namespace FlyFM
{

class TreeViewModPrivate;

class TreeViewMod : public QTreeView
{
    Q_OBJECT
public:
    explicit TreeViewMod(QWidget *parent = 0);
    ~TreeViewMod();

    QRect visualRect(const QModelIndex &index) const Q_DECL_OVERRIDE;

    void setHighlightedIndex(const QModelIndex& index);
    QModelIndex highlightedIndex() const;

    void setShowSelection(bool on);
    bool showSelection() const;


protected:
    void drawRow(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const Q_DECL_OVERRIDE;
    void drawBranches(QPainter *painter, const QRect &rect, const QModelIndex &index) const Q_DECL_OVERRIDE;
    int sizeHintForColumn(int column) const Q_DECL_OVERRIDE;

    bool viewportEvent(QEvent *event) Q_DECL_OVERRIDE;
    void mousePressEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
    void mouseReleaseEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
    void mouseDoubleClickEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
    void mouseMoveEvent(QMouseEvent *event) Q_DECL_OVERRIDE;

private:
    Q_DECLARE_PRIVATE(QTreeView)

    bool isInNoOrEditingState() const;

    friend class TreeViewModPrivate;
    QScopedPointer<TreeViewModPrivate> p;
};

}

#endif
