/*

  Graphic user's interface library
  Tree widget

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMTREEWIDGET_H
#define FLYFMTREEWIDGET_H

#include <QWidget>
#include <QModelIndexList>

#include <flyfm/entrylist.h>
#include <flyfm/imanageiconsize.h>

namespace FlyFM
{
class Navigation;
class ScheduledUpdateTreeView;
class Model;
class PathCompleter;
class IconSizeActionsProvider;
class FileActionsProvider;
class Job;
class ItemViewTouchGesturesManager;

class TreeWidget : public QWidget, public IManageIconSize
{
    Q_OBJECT
public:
    explicit TreeWidget(Navigation *navigation, QWidget *parent = 0);
    ~TreeWidget();

    void setRootEntry(const Entry &entry);
    const Entry rootEntry() const;

    Model *model();
    const Model *model() const;

    Entry currentEntry() const;

    PathCompleter* createPathCompleter();

    void setHeaderHidden(bool on);

    IconSize iconSize() const Q_DECL_OVERRIDE;

    static QByteArray defaultState(bool tabletMode);
    QByteArray saveState() const;
    void restoreState(const QByteArray& state);

    bool showHiddenEntries() const;
    void setShowHiddenEntries(bool enable);

    void setNavigation(Navigation *navigation);

    EntryList selectedEntries() const;

    void setKineticScrollingEnabled(bool on);
    void setPinchZoomEnabled(bool on);

signals:
    void entriesSelected(const EntryList &);

public slots:
    void setCurrentEntry(const Entry &entry);
    void setIconSize(IconSize size) Q_DECL_OVERRIDE;

protected slots:
    void dropPopupContextMenu(const QPoint &, const EntryList &, const Entry &);
    void customPopupContextMenu(const QPoint &, const QModelIndexList &);
    void propsDialog(const QModelIndexList &);
    void openAction(const QModelIndexList &indexes);
    void upAction();

    void increaseIconSizeAction();
    void decreaseIconSizeAction();
    void selectionChanged();

    void modelReset();
    void updateModelPresentation();

    void updateShortcutFileAction();

    void doubleClickedSlot(const QModelIndex& index);
    void clickedSlot(const QModelIndex& index);
    void enterPressedSlot();

    void updateSmbEntryJobFinished(Job* job);
    void updateNetworkPlaceJobFinished(Job* job);

protected:
    bool eventFilter(QObject* watched, QEvent* event) Q_DECL_OVERRIDE;

protected:
    Navigation *m_navigation;
    ScheduledUpdateTreeView* m_view;
    Model *m_model;
    IconSize m_iconSize;

    IconSizeActionsProvider* m_iconSizeActionsProvider;
    FileActionsProvider* m_fileActionsProvider;

    ItemViewTouchGesturesManager* m_touchGesturesManager;
};
}

#endif // FLYFMTREEWIDGET_H
